CON
  _CLKMODE = XTAL1 + PLL16X
  _CLKFREQ = 80_000_000

  One_ms = _CLKFREQ / 1000
  _stack        = 2000
  
OBJ
  NV      : "NVVar"
  rs232   : "FullDuplexSerial"
  numbers : "Numbers"


VAR
  long cacheHint[2]
  
PUB Main | t, i, fromCNT
  waitcnt(One_ms * 3000 + CNT)
  
  rs232.start(31, 30, 0, 115200)

  rs232.str(string("*** Welcome to NVVar-land! ***", 10, 10))

  if NV.Config(NV#cNVVarEEPROMAddr, $8000, NV#cNVVarSignature, false) '$8000 assumes a 24xx512 EEPROM
    rs232.str(string("Config successful, found EEPROM", 10))
  else
    rs232.str(string("Config failure", 10))

  t:= NV.IsInit 'needs first-time initialisation
  rs232.str(numbers.toStr(t, numbers#DEC))
  if not t
    rs232.str(string("Needs init", 10))
    t:= NV.Init(111)
    rs232.str(numbers.toStr(t, numbers#DEC))

  'inited now?
  if NV.IsInit
    rs232.str(string(10, "Initialized", 10))
  else
    rs232.str(string(10, "Oooops!", 10))
    
  rs232.str(string("Version-code: "))
  rs232.str(numbers.toStr(NV.GetVersion, numbers#DEC))
  rs232.tx(10)
  
  if NV.HasTag(1)
    rs232.str(string("Has tag 1"))
  else
    rs232.str(string("Has not tag 1"))
  rs232.tx(10)

  if NV.HasTag(101)
    rs232.str(string("Has tag 101"))
  else
    rs232.str(string("Has *NOT* tag 101"))
  rs232.tx(10)

  t:= 10001
  NV.WriteVar(101, @t, 4)

  t:= 10002
  NV.WriteVar(102, @t, 4)

  t:= 10003
  NV.WriteVar(103, @t, 4)
  waitcnt(One_ms * 30 + CNT)

  NV.ReadVar(101, @t, 4)
  rs232.str(numbers.toStr(t, numbers#DEC))
  rs232.tx(10)
  
  NV.ReadVar(102, @t, 4)
  rs232.str(numbers.toStr(t, numbers#DEC))
  rs232.tx(10)

  NV.ReadVar(103, @t, 4)
  rs232.str(numbers.toStr(t, numbers#DEC))
  rs232.tx(10)

  rs232.str(string(10, "Number of entries: "))
  rs232.str(numbers.toStr(NV.NumberOfEntries, numbers#DEC))
  rs232.tx(10)

  rs232.str(string(10, "Writing 300 entries:"))
  fromCNT:= CNT
  repeat i from 10000 to 10300
    NV.WriteVar(i + 500, @i, 4)
    rs232.str(string("."))
  WriteTime(fromCNT, CNT)
  
  fromCNT:= CNT
  rs232.str(string(10, "Reading 300 entries (forwards,  with forward-caching on):"))
  repeat i from 10000 to 10300
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  rs232.str(string(10, "Reading 100 entries (backwards, with forward-caching on) BE VERY PATIENT!:", 10))
  fromCNT:= CNT
  repeat i from 10100 to 10000
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  NV.SetCaching(false)
  rs232.str(string(10, "Reading 300 entries (forwards,  with forward-caching off):"))
  fromCNT:= CNT
  repeat i from 10000 to 10300
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  rs232.str(string(10, "Reading 300 entries (backwards, with forward-caching off):"))
  fromCNT:= CNT
  repeat i from 10300 to 10000
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  rs232.str(string(10, "Local read (100..200) with caching off, forward:"))
  fromCNT:= CNT
  repeat i from 10100 to 10200
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  rs232.str(string(10, "Local read (100..200) with caching off, backward:"))
  fromCNT:= CNT
  repeat i from 10200 to 10100
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  NV.SetCaching(true)
  rs232.str(string(10, "Local read (100..200) with caching on, forward:"))
  fromCNT:= CNT
  NV.ReadVar(10100 + 500, @t, 4)
  NV.GetCacheHint(@cacheHint)
  repeat i from 10100 to 10200
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)

  rs232.str(string(10, "Local read (100..200) with caching off + cache-hint, backward:"))
  NV.SetCaching(false)
  NV.SetCacheHint(@cacheHint)
  fromCNT:= CNT
  repeat i from 10200 to 10100
    NV.ReadVar(i + 500, @t, 4)
    if t <> i
      rs232.str(string("-"))
  WriteTime(fromCNT, CNT)
        
  rs232.str(string(10, "Number of entries: "))
  rs232.str(numbers.toStr(NV.NumberOfEntries, numbers#DEC))
  rs232.tx(10)

  rs232.str(string(10, "Footprint (bytes consumed in EEPROM): "))
  rs232.str(numbers.toStr(NV.GetFootprint, numbers#DEC))
  rs232.tx(10)

PRI WriteTime(fromT, toT) | time
  time:= toT - fromT  
  rs232.str(string(" Time [ms]: "))
  rs232.str(numbers.toStr(time / One_ms, numbers#DEC))
  
      